/*
 *
 * Artifactory is a binaries repository manager.
 * Copyright (C) 2018 JFrog Ltd.
 *
 * Artifactory is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 * Artifactory is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with Artifactory.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package org.artifactory.util;

import org.artifactory.version.ArtifactoryVersion;
import org.jfrog.storage.common.VersionComparator;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Objects;

/**
 * @author Dan Feldman
 */
public class ArtifactoryNodeVersionComparator implements VersionComparator {
    private static final Logger log = LoggerFactory.getLogger(ArtifactoryNodeVersionComparator.class);

    @Override
    public boolean isVersionEqual(String myVersion, String otherVersion) {
        ArtifactoryVersion myArtVersion = ArtifactoryVersion.fromVersionString(myVersion);
        ArtifactoryVersion otherArtVersion = ArtifactoryVersion.fromVersionString(otherVersion);
        logIfBlank(myVersion, otherVersion, myArtVersion, otherArtVersion);
        return Objects.equals(myArtVersion, otherArtVersion);
    }

    @Override
    public boolean isVersionBefore(String myVersion, String otherVersion) {
        ArtifactoryVersion myArtVersion = ArtifactoryVersion.fromVersionString(myVersion);
        ArtifactoryVersion otherArtVersion = ArtifactoryVersion.fromVersionString(otherVersion);
        logIfBlank(myVersion, otherVersion, myArtVersion, otherArtVersion);
        return myArtVersion != null && otherArtVersion != null && myArtVersion.before(otherArtVersion);
    }

    @Override
    public boolean isVersionAfter(String myVersion, String otherVersion) {
        ArtifactoryVersion myArtVersion = ArtifactoryVersion.fromVersionString(myVersion);
        ArtifactoryVersion otherArtVersion = ArtifactoryVersion.fromVersionString(otherVersion);
        logIfBlank(myVersion, otherVersion, myArtVersion, otherArtVersion);
        return myArtVersion != null && otherArtVersion != null && myArtVersion.after(otherArtVersion);
    }

    private void logIfBlank(String myVersion, String otherVersion, ArtifactoryVersion myArtVersion, ArtifactoryVersion otherArtVersion) {
        if (myArtVersion == null || otherArtVersion == null) {
            log.warn("Can't compare against empty version! {} <> {}", myVersion, otherVersion);
        }
    }
}
