/*
 *
 * Artifactory is a binaries repository manager.
 * Copyright (C) 2018 JFrog Ltd.
 *
 * Artifactory is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 * Artifactory is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with Artifactory.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package org.artifactory.common.storage.db.properties;

import org.apache.maven.artifact.versioning.DefaultArtifactVersion;
import org.codehaus.mojo.versions.ordering.MavenVersionComparator;

import java.util.Comparator;

/**
 * @author Gidi Shabat
 * (Originally for RTFACT-8297)
 */
public class DBPropertiesComparator implements Comparator<DbVersionInfo> {

    private final MavenVersionComparator versionComparator;

    public DBPropertiesComparator() {
        versionComparator = new MavenVersionComparator();
    }

    @Override
    public int compare(DbVersionInfo o1, DbVersionInfo o2) {
        String artifactoryVersion1 = o1.getArtifactoryVersion();
        String artifactoryVersion2 = o2.getArtifactoryVersion();
        DefaultArtifactVersion version1 = new DefaultArtifactVersion(artifactoryVersion1);
        DefaultArtifactVersion version2 = new DefaultArtifactVersion(artifactoryVersion2);
        return versionComparator.compare(version1, version2);
    }
}
