package org.artifactory.descriptor.trashcan;

import org.springframework.util.ReflectionUtils;
import org.testng.Assert;
import org.testng.annotations.Test;

import java.lang.reflect.Field;


/**
 * @author dudim
 * Retention days cannot exceed 5 years value.
 * when executing propertiesDao.getGCCandidateQuery we using less('<') comparator on strings (prop_value < validfrom).
 * we assume validFrom is with length of 10 chars (since 2002 epoch time length is 10 digits),
 * in this test we assert when setting the Retention days to over 5 years(1928 days) we will return 0
 * otherwise we will delete all trashcan content.
 */
public class TrashcanConfigDescriptorTest {

    @Test
    public void testGetRetentionPeriodDaysZeroValue() throws Exception {
        TrashcanConfigDescriptor trashcanConfigDescriptor = new TrashcanConfigDescriptor();
        setRetentionDaysReflection(trashcanConfigDescriptor, 0);
        int retentionPeriodDays = trashcanConfigDescriptor.getRetentionPeriodDays();
        Assert.assertEquals(retentionPeriodDays, 0);
    }

    @Test
    public void testGetValidRetentionPeriodDays() throws Exception {
        TrashcanConfigDescriptor trashcanConfigDescriptor = new TrashcanConfigDescriptor();
        setRetentionDaysReflection(trashcanConfigDescriptor, 15);
        int retentionPeriodDays = trashcanConfigDescriptor.getRetentionPeriodDays();
        Assert.assertEquals(retentionPeriodDays, 15);
    }

    @Test
    public void testGetRetentionPeriodDaysOver5Years() throws Exception {
        TrashcanConfigDescriptor trashcanConfigDescriptor = new TrashcanConfigDescriptor();
        setRetentionDaysReflection(trashcanConfigDescriptor, 1900);
        int retentionPeriodDays = trashcanConfigDescriptor.getRetentionPeriodDays();
        Assert.assertEquals(retentionPeriodDays, 0);
    }

    @Test
    public void testSetRetentionPeriodDays() {
        TrashcanConfigDescriptor trashcanConfigDescriptor = new TrashcanConfigDescriptor();
        trashcanConfigDescriptor.setRetentionPeriodDays(1900);
        Assert.assertEquals(trashcanConfigDescriptor.getRetentionPeriodDays(), 0);
    }

    @Test
    public void testSetValidRetentionPeriodDays() {
        TrashcanConfigDescriptor trashcanConfigDescriptor = new TrashcanConfigDescriptor();
        trashcanConfigDescriptor.setRetentionPeriodDays(1345);
        Assert.assertEquals(trashcanConfigDescriptor.getRetentionPeriodDays(), 1345);
    }

    private void setRetentionDaysReflection(TrashcanConfigDescriptor trashcanConfigDescriptor, int value)
            throws NoSuchFieldException {
        Field retentionDaysField = TrashcanConfigDescriptor.class.getDeclaredField("retentionPeriodDays");
        retentionDaysField.setAccessible(true);
        ReflectionUtils.setField(retentionDaysField, trashcanConfigDescriptor, value);
    }
}