package org.artifactory.storage.binstore.service;

import org.artifactory.common.ConstantValues;
import org.artifactory.storage.GCCandidate;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;
import java.util.function.BiConsumer;

/**
 * @author dudim
 */
public class GCProviderWrapper implements GCProvider {

    private static final Logger log = LoggerFactory.getLogger(GCProviderWrapper.class);
    private int errorCount;
    private boolean isEnabled;

    private GCProvider gcProvider;

    public GCProviderWrapper(GCProvider gcProvider) {
        this.gcProvider = gcProvider;
    }

    @Override
    public List<GCCandidate> getBatch() {
        if (isEnabled) {
            return gcProvider.getBatch();
        }
        return new ArrayList<>();
    }

    public BiConsumer<GCCandidate, GarbageCollectorInfo> getAction() {
        return (gcCandidate, result) -> {
            try {
                gcProvider.getAction().accept(gcCandidate, result);
            } catch (Exception e) {
                errorCount++;
                if (errorCount >= ConstantValues.gcFailCountThreshold.getInt()) {
                    log.error("Provider reached maximum error count: {}, disabling provider", errorCount);
                    isEnabled = false;
                }
            }
        };
    }

    public void resetProvider() {
        gcProvider.resetProvider();
        isEnabled = true;
        errorCount = 0;
    }
}