package org.artifactory.storage.db.binstore.service.garbage;

import org.artifactory.api.repo.RepositoryService;
import org.artifactory.common.ConstantValues;
import org.artifactory.storage.GCCandidate;
import org.artifactory.storage.StorageException;
import org.artifactory.storage.binstore.service.BinaryService;
import org.artifactory.storage.binstore.service.GCProvider;
import org.artifactory.storage.binstore.service.GarbageCollectorInfo;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;
import java.util.function.BiConsumer;

/**
 * @author dudim
 */
public class BinariesGCProvider implements GCProvider {
    private static final Logger log = LoggerFactory.getLogger(TrashGCProvider.class);

    private BinaryService binaryService;
    private RepositoryService repositoryService;
    private GCStreamingQueryHelper gcStreamingQueryHelper;

    public BinariesGCProvider(BinaryService binaryService, RepositoryService repositoryService) {
        this.binaryService = binaryService;
        this.repositoryService = repositoryService;
        gcStreamingQueryHelper = new GCStreamingQueryHelper(ConstantValues.trashcanMaxSearchResults.getInt(),
                binaryService::getBinariesGCCandidatesStreaming, this::gcCandidateFromResultSet);
    }

    @Override
    public List<GCCandidate> getBatch() {
        return gcStreamingQueryHelper.nextBatch();
    }

    @Override
    public BiConsumer<GCCandidate, GarbageCollectorInfo> getAction() {
        return (gcCandidate, result) -> {
            if (!ConstantValues.gcBinariesJoinWithNodesQuery.getBoolean() &&
                    repositoryService.existsBySha1(gcCandidate.getSha1())) {
                log.info("Binary: {} exist, will not be deleted", gcCandidate.getSha1());
                return;
            }
            binaryService.executeBinaryCleaner(gcCandidate, result);
        };
    }

    @Override
    public void resetProvider() {
        gcStreamingQueryHelper.reset();
    }

    private GCCandidate gcCandidateFromResultSet(ResultSet resultSet) {
        try {
            String sha1 = resultSet.getString("sha1");
            String sha2 = resultSet.getString("sha256");
            String md5 = resultSet.getString("md5");
            String length = resultSet.getString("bin_length");
            return new GCCandidate(null, sha1, sha2, md5, Long.valueOf(length));
        } catch (SQLException e) {
            throw new StorageException("Unable to read GC Candidate from result set", e);
        }
    }
}
