package org.artifactory.storage.db.binstore.service.garbage;

import org.artifactory.common.ConstantValues;
import org.artifactory.repo.RepoPath;
import org.artifactory.repo.trash.TrashService;
import org.artifactory.storage.GCCandidate;
import org.artifactory.storage.binstore.service.GCProvider;
import org.artifactory.storage.binstore.service.GarbageCollectorInfo;
import org.artifactory.storage.db.fs.dao.PropertiesDao;
import org.jfrog.storage.StorageException;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;
import java.util.function.BiConsumer;

/**
 * @author dudim
 */
public class TrashGCProvider implements GCProvider {
    private GCStreamingQueryHelper gcStreamingQueryHelper;
    private TrashService trashService;

    public TrashGCProvider(TrashService trashService) {
        this.trashService = trashService;
        gcStreamingQueryHelper = new GCStreamingQueryHelper(ConstantValues.trashcanMaxSearchResults.getInt(),
                trashService::getGCCandidatesFromTrashStreaming, this::gcCandidateFromResultSet);
    }

    @Override
    public List<GCCandidate> getBatch() {
        return gcStreamingQueryHelper.nextBatch();
    }

    @Override
    public BiConsumer<GCCandidate, GarbageCollectorInfo> getAction() {
        return (gcCandidate, result) -> trashService.undeployFromTrash(gcCandidate);
    }

    @Override
    public void resetProvider() {
        gcStreamingQueryHelper.reset();
    }

    private GCCandidate gcCandidateFromResultSet(ResultSet resultSet) {
        try {
            RepoPath repoPath = PropertiesDao.repoPathFromResultSet(resultSet);
            String sha1 = resultSet.getString("sha1_actual");
            String sha2 = resultSet.getString("sha256");
            String md5 = resultSet.getString("md5_actual");
            String length = resultSet.getString("bin_length");
            return new GCCandidate(repoPath, sha1, sha2, md5, Long.valueOf(length));
        } catch (SQLException e) {
            throw new StorageException("Unable to read GC Candidate from result set", e);
        }
    }
}
